@@ -1,6 +1,7 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* Copyright 2008, 2009 Linas Vepstas                                    */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
@@ -1248,6 +1249,14 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 		if (rc) return -1;
 	}
 
+	/* Check for bad sentence length */
+	if (MAX_SENTENCE <= sent->length)
+	{
+		prt_error("Error: sentence too long, contains more than %d words\n", 
+			MAX_SENTENCE);
+		return -2;
+	}
+
 	/* Initialize/free any leftover garbage */
 	free_sentence_disjuncts(sent);
 	resources_reset_space(opts->resources);
