@@ -1,6 +1,7 @@
 /***************************************************************************/
 /* Copyright (c) 2004                                                      */
 /* Daniel Sleator, David Temperley, and John Lafferty                      */
+/* Copyright (c) 2008 Linas Vepstas                                        */
 /* All rights reserved                                                     */
 /*                                                                         */
 /* Use of the link grammar parsing system is subject to the terms of the   */
@@ -40,6 +41,14 @@
 #include <string.h>
 #include <wchar.h>
 
+/* Used for terminal resizing */
+#ifndef _MSC_VER
+#include <termios.h>
+#include <sys/ioctl.h>
+#include <fcntl.h>
+#include <unistd.h>
+#endif
+
 #ifdef HAVE_EDITLINE
 #include <editline/readline.h>
 #endif 
@@ -448,6 +457,38 @@ static void print_usage(char *str) {
 	exit(-1);
 }
 
+/**
+ * On Unix, this checks for the current window size, 
+ * and sets the output screen width accordingly. 
+ * Not sure how MS Windows does this.
+ */
+static void check_winsize(Parse_Options popts)
+{
+#ifndef _MSC_VER
+	struct winsize ws;
+	int fd = open("/dev/tty",O_RDWR);
+
+	if (0 != ioctl(fd,TIOCGWINSZ, &ws))
+	{
+		perror("ioctl(/dev/tty,TIOCGWINSZ)");
+		close (fd);
+		return;
+	}
+	close (fd);
+
+	/* printf("rows %i\n", ws.ws_row); */
+	/* printf("cols %i\n", ws.ws_col); */
+
+	/* Set the screen width only if the returned value seems
+	 * rational: its positive and not insanely tiny.
+	 */
+	if ((10 < ws.ws_col) && (16123 > ws.ws_col))
+	{
+		parse_options_set_screen_width(popts, ws.ws_col - 1);
+	}
+#endif /* _MSC_VER */
+}
+
 int main(int argc, char * argv[])
 {
 	FILE            *input_fh = stdin;
@@ -462,6 +503,16 @@ int main(int argc, char * argv[])
 	Label           label = NO_LABEL;
 	const char      *codeset;
 
+#if LATER
+	/* Try to catch the SIGNWINCH ... except this is not working. */
+	struct sigaction winch_act;
+	winch_act.sa_handler = winch_handler;
+	winch_act.sa_sigaction = NULL;
+	sigemptyset (&winch_act.sa_mask);
+	winch_act.sa_flags = 0;
+	sigaction (SIGWINCH, &winch_act, NULL);
+#endif
+
 	i = 1;
 	if ((argc > 1) && (argv[1][0] != '-')) {
 		/* the dictionary is the first argument if it doesn't begin with "-" */
@@ -480,6 +531,9 @@ int main(int argc, char * argv[])
 	codeset = nl_langinfo(CODESET);
 	if (!strstr(codeset, "UTF") && !strstr(codeset, "utf"))
 	{
+		fprintf(stderr, 
+		    "%s: Warning: locale %s was not UTF-8; force-setting to en_US.UTF-8\n",
+		     argv[0], codeset);
 		setlocale(LC_CTYPE, "en_US.UTF-8");
 	}
 
@@ -552,6 +606,7 @@ int main(int argc, char * argv[])
 	}
 
 	verbosity = parse_options_get_verbosity(opts);
+	check_winsize(opts);
 
 	prt_error("Info: Version %s. Enter \"!help\" for help.\n",
 		linkgrammar_get_version());
@@ -560,6 +615,7 @@ int main(int argc, char * argv[])
 	while (1)
 	{
 		input_string = fget_input_string(input_fh, stdout, opts);
+		check_winsize(opts);
 
 		if (NULL == input_string)
 		{
