@@ -263,7 +263,7 @@ static int issue_sentence_word(Sentence sent, const char * s)
 		return FALSE;
 	}
 
-	if (sent->length == MAX_SENTENCE)
+	if (sent->length >= MAX_SENTENCE)
 	{
 		err_ctxt ec;
 		ec.sent = sent;
@@ -338,6 +338,17 @@ static int issue_sentence_word(Sentence sent, const char * s)
 #undef		MIN
 #define MIN(a, b)  (((a) < (b)) ? (a) : (b))
 
+static int boolean_reg_dict_lookup(Dictionary dict, const char * word)
+{
+	const char * regex_name;
+	if (boolean_dictionary_lookup(dict, word)) return TRUE;
+
+	regex_name = match_regex(dict, word);
+	if (NULL == regex_name) return FALSE;
+
+	return boolean_dictionary_lookup(dict, regex_name);
+}
+
 static int downcase_is_in_dict(Dictionary dict, char * word)
 {
 	int i, rc;
@@ -365,7 +376,7 @@ static int downcase_is_in_dict(Dictionary dict, char * word)
 	for (i=0; i<nbl; i++) { save[i] = word[i]; word[i] = low[i]; }
 
 	/* Look it up, then restore old value */
-	rc = boolean_dictionary_lookup(dict, word);
+	rc = boolean_reg_dict_lookup(dict, word);
 	for (i=0; i<nbh; i++) { word[i] = save[i]; }
 
 	return rc; 
@@ -376,12 +387,13 @@ static int downcase_is_in_dict(Dictionary dict, char * word)
  * "word" w contains no blanks.  This function splits up the word if
  * necessary, and calls "issue_sentence_word()" on each of the resulting
  * parts.  The process is described above.  Returns TRUE if OK, FALSE if
- * too many punctuation marks 
+ * too many punctuation marks or other separation error.
  */
 static int separate_word(Sentence sent, Parse_Options opts,
                          const char *w, const char *wend,
                          int is_first_word, int quote_found)
 {
+	size_t sz;
 	int i, j, len;
 	int r_strippable=0, l_strippable=0, u_strippable=0;
 	int s_strippable=0, p_strippable=0;
@@ -406,11 +418,12 @@ static int separate_word(Sentence sent, Parse_Options opts,
 	/* First, see if we can already recognize the word as-is. If
 	 * so, then we are done. Else we'll try stripping prefixes, suffixes.
 	 */
-	strncpy(word, w, MIN(wend-w, MAX_WORD));
-	word[MIN(wend-w, MAX_WORD)] = '\0';
+	sz = MIN(wend-w, MAX_WORD);
+	strncpy(word, w, sz);
+	word[sz] = '\0';
 	word_is_in_dict = FALSE;
 
-	if (boolean_dictionary_lookup(sent->dict, word))
+	if (boolean_reg_dict_lookup(sent->dict, word))
 		word_is_in_dict = TRUE;
 	else if (is_initials_word(word))
 		word_is_in_dict = TRUE;
@@ -419,8 +432,7 @@ static int separate_word(Sentence sent, Parse_Options opts,
 
 	if (word_is_in_dict)
 	{
-		issue_sentence_word(sent, word);
-		return TRUE;
+		return issue_sentence_word(sent, word);
 	}
 
 	/* Set up affix tables.  */
@@ -446,16 +458,23 @@ static int separate_word(Sentence sent, Parse_Options opts,
 		for (i=0; i<l_strippable; i++)
 		{
 			/* This is UTF8-safe, I beleive ... */
-			if (strncmp(w, strip_left[i], strlen(strip_left[i])) == 0)
+			sz = strlen(strip_left[i]);
+			if (strncmp(w, strip_left[i], sz) == 0)
 			{
 				if (!issue_sentence_word(sent, strip_left[i])) return FALSE;
-				w += strlen(strip_left[i]);
+				w += sz;
 				break;
 			}
 		}
 		if (i == l_strippable) break;
 	}
 
+	/* Its possible that the token consisted entirely of
+	 * left-punctuation, in which case, it has all been issued.
+	 * So -- we're done, return.
+	 */
+	if (w >= wend) return TRUE;
+
 	/* Now w points to the string starting just to the right of
 	 * any left-stripped characters.
 	 * stripped[] is an array of numbers, indicating the index
@@ -466,12 +485,12 @@ static int separate_word(Sentence sent, Parse_Options opts,
 	 */
 	for (n_r_stripped = 0; n_r_stripped < MAX_STRIP; n_r_stripped++) 
 	{
-		size_t sz = MIN(wend-w, MAX_WORD);
+		sz = MIN(wend-w, MAX_WORD);
 		strncpy(word, w, sz);
 		word[sz] = '\0';
 		if (wend == w) break;  /* it will work without this */
 
-		if (boolean_dictionary_lookup(sent->dict, word) ||
+		if (boolean_reg_dict_lookup(sent->dict, word) ||
 		    is_initials_word(word)) break;
 
 		/* This could happen if it's a word after a colon, also! */
@@ -552,7 +571,7 @@ static int separate_word(Sentence sent, Parse_Options opts,
 	word[MIN(wend-w, MAX_WORD)] = '\0';
 	word_is_in_dict = FALSE;
 
-	if (boolean_dictionary_lookup(sent->dict, word))
+	if (boolean_reg_dict_lookup(sent->dict, word))
 		word_is_in_dict = TRUE;
 	else if (is_initials_word(word))
 		word_is_in_dict = TRUE;
@@ -585,7 +604,7 @@ static int separate_word(Sentence sent, Parse_Options opts,
 
 				/* Check if the remainder is in the dictionary;
 				 * for the no-suffix case, it won't be */
-				if (boolean_dictionary_lookup(sent->dict, newword))
+				if (boolean_reg_dict_lookup(sent->dict, newword))
 				{
 					if ((verbosity>1) && (i < s_strippable))
 						printf("Splitting word into two: %s-%s\n", newword, suffix[i]);
@@ -608,7 +627,7 @@ static int separate_word(Sentence sent, Parse_Options opts,
 							int sz = MIN((wend-len)-(w+strlen(prefix[j])), MAX_WORD);
 							strncpy(newword, w+strlen(prefix[j]), sz);
 							newword[sz] = '\0';
-							if (boolean_dictionary_lookup(sent->dict, newword))
+							if (boolean_reg_dict_lookup(sent->dict, newword))
 							{
 								if ((verbosity>1) && (i < s_strippable))
 									printf("Splitting word into three: %s-%s-%s\n",
@@ -632,12 +651,16 @@ static int separate_word(Sentence sent, Parse_Options opts,
 	}
 
 	/* word is now what remains after all the stripping has been done */
+	issued = FALSE;
 
+	/* If n_r_stripped exceed max, the "word" is most likely a long
+	 * sequence of periods.  Just accept it as an unknown "word", 
+	 * and move on.
+	 */
 	if (n_r_stripped >= MAX_STRIP)
 	{
-		prt_error("Error separating sentence.\n"
-		          "\"%s\" is followed by too many punctuation marks.\n", word);
-		return FALSE;
+		n_r_stripped = 0;
+		word_is_in_dict = TRUE;
 	}
 
 	if (quote_found == TRUE) sent->post_quote[sent->length] = 1;
@@ -647,7 +670,6 @@ static int separate_word(Sentence sent, Parse_Options opts,
 	 * the word in two, if possible. Do this only if the word
 	 * is not a proper name, and if spell-checking is enabled.
 	 */
-	issued = FALSE;
 	if ((FALSE == word_is_in_dict) && 
 	    TRUE == opts->use_spell_guess &&
 	    sent->dict->spell_checker &&
@@ -671,12 +693,12 @@ static int separate_word(Sentence sent, Parse_Options opts,
 		while (sp)
 		{
 			*sp = 0x0;
-			issue_sentence_word(sent, wp);
+			if (!issue_sentence_word(sent, wp)) return FALSE;
 			wp = sp+1;
 			sp = strchr(wp, ' ');
 			if (NULL == sp)
 			{
-				issue_sentence_word(sent, wp);
+				if (!issue_sentence_word(sent, wp)) return FALSE;
 			}
 		}
 		if (alternates) free(alternates);
@@ -806,7 +828,7 @@ static int special_string(Sentence sent, int i, const char * s)
 
 /**
  * Build the word expressions, and add a tag to the word to indicate
- * that it was guess by means of regular-expression matching.
+ * that it was guessed by means of regular-expression matching.
  * Also, add a subscript to the resulting word to indicate the
  * rule origin.
  */
@@ -963,7 +985,7 @@ static void guess_misspelled_word(Sentence sent, int i, char * s)
 	n = spellcheck_suggest(dict->spell_checker, &alternates, s);
 	for (j=0; j<n; j++)
 	{
-		if (boolean_dictionary_lookup(sent->dict, alternates[j]))
+		if (boolean_reg_dict_lookup(sent->dict, alternates[j]))
 		{
 			X_node *x = build_word_expressions(sent->dict, alternates[j]);
 			head = catenate_X_nodes(x, head);
@@ -1153,7 +1175,7 @@ int build_sentence_expressions(Sentence sent, Parse_Options opts)
 			downcase_utf8_str(temp_word, s, MAX_WORD);
 			lc = string_set_add(temp_word, sent->string_set);
 
-			if (boolean_dictionary_lookup(sent->dict, lc))
+			if (boolean_reg_dict_lookup(sent->dict, lc))
 			{
 				if (is_entity(sent->dict,s) ||
 				    is_common_entity(sent->dict,lc))
@@ -1182,6 +1204,8 @@ int build_sentence_expressions(Sentence sent, Parse_Options opts)
  * up an appropriate error message in case some are not there.
  * It has no side effect on the sentence.  Returns TRUE if all
  * went well.
+ *
+ * This code is called only is the 'unkown-words' flag is set.
  */
 int sentence_in_dictionary(Sentence sent)
 {
@@ -1194,10 +1218,7 @@ int sentence_in_dictionary(Sentence sent)
 	for (w=0; w<sent->length; w++)
 	{
 		s = sent->word[w].string;
-		/* xx we should also check if the regex rules know this word */
-		/* However, this code is only used if the 'unknown word' is 
-		 * turned off ... we punt for now, since its always on. */
-		if (!boolean_dictionary_lookup(dict, s))
+		if (!boolean_reg_dict_lookup(dict, s))
 #if DONT_USE_REGEX_GUESSING
 		    !(is_utf8_upper(s)   && dict->capitalized_word_defined) &&
 		    !(is_utf8_upper(s) && is_s_word(s) && dict->pl_capitalized_word_defined) &&
